<?php

require_once( 'FrogEnvoy/Models/AbstractApacheConf.php' );
require_once( 'FrogEnvoy/Models/AbstractApacheConfPathValue.php' );

class FrogEnvoy_Models_ApacheConfFile extends FrogEnvoy_Models_AbstractApacheConf
	{
	protected $includedConfObjs = array();
	


	public function __construct( $filePath )
		{
		$this->_filePath = $filePath;
		}



	public function findListOfIncludedFilePaths()
		{		
		$listOfIncludedFilePaths = $this->findListOfIncludeFilePathsAsAbsoluteValues();

		$fileListArray = array();
		foreach( $listOfIncludedFilePaths as $path )
			{
			$pathValue = FrogEnvoy_Models_AbstractApacheConfPathValue::createConfIncludePath( $path );

			if (!( $pathValue->isPathToSingleFile() ))
				{
				foreach( $pathValue->getExpandedFileListArray() as $path )
					{
					$fileListArray[] = $path;
					}
				}
			else	{
				$fileListArray[] = $pathValue->getPathValue();
				}
			}
		return $fileListArray;
		}



	protected function findListOfIncludeFilePathsAsAbsoluteValues()
		{
		$serverRootAndIncludeDirectiveLines = $this->findAllServerRootAndIncludeDirectiveLinesInSequence();

		$absoluteIncludeFilePathsArray = array();

		foreach( $serverRootAndIncludeDirectiveLines as $line )
			{
			if ( $this->isServerRootDirectiveLine( $line ) )
				{

				//save in case we need it 
				$latestServerRootDirectiveValue
					= $this->unQuoteCString( 
						$this->extractTrailingValueFromDirectiveLine(
							'ServerRoot', 
							$line 
							) );

				//echo 'found server root directive: ' .$latestServerRootDirectiveValue;
				continue;
				}

			//else it must be an include directive - get the value
			$absoluteIncludeFilePathsArray[]
				= $this->getAsAbsolutePath( 
					$latestServerRootDirectiveValue,
					$this->unQuoteCString( 
						$this->extractTrailingValueFromDirectiveLine( 
							'Include', 
							$line ) 
						)
					);
			}

		return $absoluteIncludeFilePathsArray;
		}

	protected function getAsAbsolutePath( $serverRoot, $path )
		{

		if (!( $this->isAbsolutePath( $path ) ))
			{
			//it is therefore server root relative
			trigger( 
				'warning - we are not actually clever enough to know if 
				the server root prefix applies to this include path.', 
				E_USER_WARNING 
				);
			return $path = $serverRoot . '/' . $path;
			}
		
		return $path;
		}

	/**
	* @desc in apache config anything the begins with a forward slash is treated as absolute path
	* See: http://httpd.apache.org/docs/2.0/mod/core.html#include
	*/
	protected function isAbsolutePath( $line )
		{
		return ( strpos( trim( $line ) , '/' ) === 0 );
		}


	protected function isServerRootDirectiveLine( $line )
		{
		return ( strpos( trim( $line ) , 'ServerRoot' ) === 0 );
		}

	protected function findAllServerRootAndIncludeDirectiveLinesInSequence()
		{
		return $this->findAllLinesMatchingDirectives( array( 'Include', 'ServerRoot' ) );
		}

	protected function getCommandCatAllNonCommentedAndNonWhitespaceLines()
		{
		return ' cat '. escapeshellcmd( $this->_filePath ) .' ' //read contents to stdout
			. ' | grep  '			
			. '-e \'^[:space:]*#|^[:space:]*$\' ' //match commented lines and lines of whitespace
			. '-v'; //Invert the sense of matching, to select non-matching lines.  (-v is specified by POSIX.)
		}
	
	/**
	* @desc 
	* Examples:
	* Include /usr/local/apache2/conf/ssl.conf
	* Include /usr/local/apache2/conf/vhosts/*.conf 
	*/
	protected function findAllIncludeDirectiveValues()
		{
		$directiveName = 'Include';
		return $this->findAllValuesOfDirective( $directiveName );
		}


	/**
	* @desc 
	* Examples:
	* ServerRoot directory-path	
	*/
	protected function findAllServerRootValues()
		{
		$directiveName = 'ServerRoot';
		return $this->findAllValuesOfDirective( $directiveName );
		}

	protected function extractTrailingValueFromDirectiveLine( $directiveName, $line )
		{		
		return trim( substr( trim( $line ), ( strlen( trim( $directiveName ) ) ) ) );
		}



	/**
	* @desc 
	* ServerRoot directory-path
	* @return array of unquoted values
	*/
	protected function findAllValuesOfDirective( $directiveName )
		{
		
		$linesArray = $this->findAllLinesMatchingDirectives( array( $directiveName ) );

		$valuesArray = array();
		foreach( $linesArray as $line )
			{
			$valuesArray[] = $this->unQuoteCString( $this->extractTrailingValueFromDirectiveLine( $directiveName, $line ) );
			}
		return $valuesArray;
		}



	/**
	* @desc 
	* 
	*/
	protected function findAllLinesMatchingDirectives( $directiveNamesArray )
		{

		if (!( $this->getCommandCatAllNonCommentedAndNonWhitespaceLines() ))
			{
			return null;
			}

		//build the or pattern
		$infixOperator = '';
		$pattern = '';
		foreach ( $directiveNamesArray as $directiveName )
			{
			//match non/some white space followed by directive name
			$pattern .= $infixOperator . '^[:space:]*'. escapeshellcmd( $directiveName );
			$infixOperator = '|';
			}

		//var_dump( $this->findAllNonCommentedAndNonWhitespaceLines() );
		//exit;

		$command = $this->getCommandCatAllNonCommentedAndNonWhitespaceLines()
			. ' | grep '			
			. '-i -E \''. $pattern  .'\' '; 
			
		//echo $command;
		//exit;


		$lastLine = exec ( 
			$command,
			$outputLines
			);

		return $outputLines;
		//note might find paths with wildcards etc		
		}


	}


